#!/bin/env python
"""
Usage: ./thisProg.py mode inFile.dp inFile.struct > outFile.txt
where mode is 1, 2, or 3, for ssRNA probabilities, base pair probabilities,
or 'base pairing statistic'.
"""

import sys

if len (sys.argv) != 4:
    print >> sys.stderr, "Need exactly 4 args.  See usage in code."
    sys.exit (1)
else:
    mode = int (sys.argv[1])
    if (mode != 1) and (mode != 2) and (mode != 3):
        print >> sys.stderr, \
            "Invalid arg value (must be 1, 2, or 3); you gave: %d" \
            % mode
        sys.exit (1)
    inFileDp     = sys.argv[2]
    inFileStruct = sys.argv[3]

# Load base pairs.
struct = [l.strip() for l in open (inFileStruct, 'r')][2]
bp     = []  # list of base pairs as tuples, 0-based indexing
stack  = []
for i in xrange (len (struct)):
    if struct[i] == '(':
        stack.append (i)
    elif struct[i] == ')':
        bp.append ((stack.pop(), i))
assert len (stack) == 0

inputLines = [l for l in open (inFileDp, 'r')]

seqLen = int (inputLines[0])
assert seqLen > 0
print >> sys.stderr, 'Seq len = %d' % seqLen

# Init with p=0 everywhere.
bpProbs = [
    [0. for j in xrange (seqLen)]
    for i in xrange (seqLen)]

# Load base pair probabilities from dot plot.
for lineNum in xrange (2, len (inputLines)):
    fields = inputLines[lineNum].strip().split()
    assert len (fields) == 3
    # i,j are 1-based base indices of the base pairs (i < j)
    # 3rd field is -log10 (p)
    i = int (fields[0])
    j = int (fields[1])
    assert i < j
    p = 10. ** (-float (fields[2]))
    assert p >= 0. and p <= 1., \
        'Invalid probability on line %d: p(%d,%d is bp) = %f' \
        % (lineNum+1, i, j, p)
    #print >> sys.stderr, i, j, p, len (bpProbs), len (bpProbs[i])
    bpProbs[i-1][j-1] = p
    bpProbs[j-1][i-1] = p  # symmetry is convenient later

# Sanity check data loaded fine.
#for i in xrange (seqLen):
#    for j in xrange (i+1, seqLen):
#        print 'p(%d,%d is bp) = %f' % (i+1, j+1, bpProbs[i][j])

if mode == 1:
    # Print ssRNA probabilities color code in VARNA form.
    ssRnaProbs = []
    for i in xrange (seqLen):
        ssRnaProbs.append (1. - sum (bpProbs[i]))
    print '<param name="colorMap" value="%s" />' \
        % ';'.join (['%.2f' % f for f in ssRnaProbs])
    print '<param name="colorMapCaption" value="p(base is ssRNA)" />'
    print '<param name="colorMapStyle" value="blue" />'

elif mode == 2:
    # Print base pair probabilities color code in VARNA form.
    prPerBase = [0. for i in xrange (seqLen)]  # index is base num in 0-based
    for i, j in bp:
        assert prPerBase[i] == 0.
        assert prPerBase[j] == 0.
        prPerBase[i] = bpProbs[i][j]
        prPerBase[j] = bpProbs[i][j]
    print '<param name="colorMap" value="%s" />' \
        % ';'.join (['%.2f' % f for f in prPerBase])
    print '<param name="colorMapCaption" value="p(this base pair)" />'
    print '<param name="colorMapStyle" value="red" />'

elif mode == 3:
    assert False, "TODO: Implement!!!"

else:
    assert False, 'Unreachable.'


